part of mvu_flutter;

/// Root [Program] widget.
///
/// Works essentially as Elm's version, but model and
/// commands arguments are split to reduce boilerplate because of Dart's
/// absence of algebraic data types.
class Program<ModelType extends Model, MsgType extends Msg>
    extends StatefulWidget {
  final ModelType model;
  final InitialView<ModelType, MsgType> view;
  final Update<ModelType, MsgType> update;
  final List<Cmd<ModelType, MsgType>>? commands;
  final Subscriptions<ModelType, MsgType>? subscriptions;

  Program(
      {required this.model,
      required this.view,
      required this.update,
      this.commands,
      this.subscriptions});

  _ProgramState<ModelType, MsgType> createState() =>
      _ProgramState<ModelType, MsgType>();
}

class _ProgramState<ModelType extends Model, MsgType extends Msg>
    extends State<Program> {
  late final _Manager<ModelType, MsgType> manager;

  @override
  void initState() {
    super.initState();
    manager = _Manager(
      model: widget.model as ModelType,
      updater: widget.update as Update<ModelType, MsgType>,
      commands: widget.commands as List<Cmd<ModelType, MsgType>>?,
      subscriptions: widget.subscriptions as Subscriptions<ModelType, MsgType>?
    );
    _Runtime.addManager(manager);
  }

  @override
  void dispose() async {
    super.dispose();
    await _Runtime.removeManager(manager);
  }

  Widget build(BuildContext context) {
    manager.model.setMarker(context);
    return widget.view();
  }
}
