part of mvu_flutter;

/// [Program] [Manager]. Only mutable part that is not accessed by package's user.
///
/// Stores model and updater, used by [dispatch]/[dispatchGlobal] to receive
/// [Msg]s, update model accordingly and execute commands.
class _Manager<ModelType extends Model, MsgType extends Msg> {
  final _Rebuilder<ModelType> model;
  final Update<ModelType, MsgType> updater;
  final Subscriptions<ModelType, MsgType>? subscriptions;
  List<StreamSubscription>? _subscriptions;

  _Manager(
      {required model,
      required this.updater,
      required commands,
      required this.subscriptions})
      : model = _Rebuilder(model) {
    _executeCommands(commands);
    if (subscriptions != null) {
      _subscriptions = subscriptions!(update);
    }
  }

  Future<void> update(MsgType message) async {
    final producedState = updater.update(message, model.value);
    model.value = producedState.model;
    await _executeCommands(producedState.commands);
  }

  Future<void> _executeCommands(List<Cmd<ModelType, MsgType>>? commands) async {
    if (commands != null) {
      for (var command in commands) {
        await command(update);
      }
    }
  }

  Future<void> dispose() async {
    model.dispose();
    if (_subscriptions != null) {
      for (var subscription in _subscriptions!) {
        await subscription.cancel();
      }
    }
  }
}
