part of mvu_flutter;

/// Static [Manager]s' store, used to access specified Manager by its type.
///
/// [getManager] call will access the last added Manager of given type if
/// several are being used.
abstract class _Runtime {
  static ListQueue<_Manager> _managers = ListQueue();

  static void addManager(_Manager manager) => _managers.addLast(manager);

  static _Manager<ModelType, MsgType>?
      getManager<ModelType extends Model, MsgType extends Msg>() {
    final foundManagers = _managers.whereType<_Manager<ModelType, MsgType>>();
    if (foundManagers.isNotEmpty) return foundManagers.last;
  }

  static Future<void> removeManager(_Manager manager) async {
    await manager.dispose();
    _managers.remove(manager);
  }

  static void isProgramExists<ModelType extends Model, MsgType extends Msg>() {
    if (getManager<ModelType, MsgType>() == null) {
      throw Exception("There is no corresponding Program of model $ModelType");
    }
  }
}
